/**
 * @license
 * Copyright 2018 The Lighthouse Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS-IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';
/* globals self DOM PerformanceCategoryRenderer Util I18n DetailsRenderer */

/**
 * Returns all the elements that PSI needs to render the report
 * We expose this helper method to minimize the 'public' API surface of the renderer
 * and allow us to refactor without two-sided patches.
 *
 *   const {scoreGaugeEl, perfCategoryEl, finalScreenshotDataUri} = prepareLabData(
 *      LHResultJsonString,
 *      document
 *   );
 *
 * @param {LH.Result | string} LHResult The stringified version of {LH.Result}
 * @param {Document} document The host page's window.document
 * @return {{scoreGaugeEl: Element, perfCategoryEl: Element, finalScreenshotDataUri: string|null, scoreScaleEl: Element}}
 */

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function prepareLabData(LHResult, document) {
  var lhResult = typeof LHResult === 'string' ?
  /** @type {LH.Result} */
  JSON.parse(LHResult) : LHResult;
  var dom = new DOM(document); // Assume fresh styles needed on every call, so mark all template styles as unused.

  dom.resetTemplates();
  var reportLHR = Util.prepareReportResult(lhResult);
  var i18n = new I18n(reportLHR.configSettings.locale, _objectSpread(_objectSpread({}, Util.UIStrings), reportLHR.i18n.rendererFormattedStrings));
  Util.i18n = i18n;
  var perfCategory = reportLHR.categories.performance;
  if (!perfCategory) throw new Error("No performance category. Can't make lab data section");
  if (!reportLHR.categoryGroups) throw new Error("No category groups found."); // Use custom title and description.

  reportLHR.categoryGroups.metrics.title = Util.i18n.strings.labDataTitle;
  reportLHR.categoryGroups.metrics.description = Util.i18n.strings.lsPerformanceCategoryDescription;
  var perfRenderer = new PerformanceCategoryRenderer(dom, new DetailsRenderer(dom)); // PSI environment string will ensure the categoryHeader and permalink elements are excluded

  var perfCategoryEl = perfRenderer.render(perfCategory, reportLHR.categoryGroups, 'PSI');
  var scoreGaugeEl = dom.find('.lh-score__gauge', perfCategoryEl);
  scoreGaugeEl.remove();
  var scoreGaugeWrapperEl = dom.find('.lh-gauge__wrapper', scoreGaugeEl);
  scoreGaugeWrapperEl.classList.add('lh-gauge__wrapper--huge'); // Remove navigation link on gauge

  scoreGaugeWrapperEl.removeAttribute('href');

  var finalScreenshotDataUri = _getFinalScreenshot(perfCategory);

  var clonedScoreTemplate = dom.cloneTemplate('#tmpl-lh-scorescale', dom.document());
  var scoreScaleEl = dom.find('.lh-scorescale', clonedScoreTemplate);
  return {
    scoreGaugeEl: scoreGaugeEl,
    perfCategoryEl: perfCategoryEl,
    finalScreenshotDataUri: finalScreenshotDataUri,
    scoreScaleEl: scoreScaleEl
  };
}
/**
 * @param {LH.ReportResult.Category} perfCategory
 * @return {null|string}
 */


function _getFinalScreenshot(perfCategory) {
  var auditRef = perfCategory.auditRefs.find(function (audit) {
    return audit.id === 'final-screenshot';
  });
  if (!auditRef || !auditRef.result || auditRef.result.scoreDisplayMode === 'error') return null;
  return (
    /** @type {LH.Audit.Details.Screenshot} */
    auditRef.result.details.data
  );
}

var _default = prepareLabData;
exports.default = _default;